#!/usr/bin/env python3

"""
Copyright or © or Copr. University of Nantes / LS2N (2021)

Contributor:
* Alexis Bitaillou

This software is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use,
modify and/ or redistribute the software under the terms of the CeCILL-C
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info".

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability.

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or
data to be ensured and,  more generally, to use and operate it in the
same conditions as regards security.

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.
"""

import pathlib
import re
from typing import Union, List

import svgwrite


class Node:
    def __init__(self, id: int, x: float, y: float, z: float):
        self.id = id
        self.x = x
        self.y = y
        self.z = z


def read_nodes(filename: Union[str, pathlib.Path]) -> List[Node]:
    nodes = list()
    with open(filename, 'r', encoding='utf8') as file:
        data = file.read()
    pattern = re.compile(r"(\d+) \d+ \((\d+\.?\d*), (\d+\.?\d*), (\d+\.?\d*)\) \d+ \d+ \d+")
    results = re.findall(pattern, data)
    for row in results:
        nodes.append(Node(int(row[0]), float(row[1]), float(row[2]), float(row[3])))
    return nodes


def node_drawer_factory(node: Node, dwg: svgwrite.Drawing):
    dwg.add(dwg.circle(center=(node.x, node.y),
                       r=20,
                       stroke=svgwrite.rgb(0, 0, 0, '%'),
                       fill='white'))
    dwg.add(
        dwg.text(str(node.id), insert=(node.x - 5.5 - len(str(node.id)), node.y + 4.5), stroke='none',
                 fill=svgwrite.rgb(15, 15, 15, '%'),
                 font_size='15px', font_style='oblique'))


if __name__ == "__main__":
    dwg = svgwrite.Drawing(filename="grid94.svg", profile='tiny')
    dwg.add(dwg.rect((0, 0), (1000, 1000), stroke=svgwrite.rgb(10, 10, 10, '%'), fill='none'))
    tmp = read_nodes("grid94.nodes")
    for node in tmp:
        node_drawer_factory(node, dwg)
    dwg.save()
