"""
Copyright or © or Copr. University of Nantes / LS2N (2021)

Contributor:
* Alexis Bitaillou

This software is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use,
modify and/ or redistribute the software under the terms of the CeCILL-C
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info".

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability.

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or
data to be ensured and,  more generally, to use and operate it in the
same conditions as regards security.

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.
"""

import os
import sqlite3
from typing import Callable

fallback_mariadb = False
try:
    from mariadb import _mariadb, Error, connect, DatabaseError
except ImportError:
    print("mariadb connector not found.")
    fallback_mariadb = True


    class _mariadb:
        connection = None
        IntegrityError = Exception


    class Error(Exception):
        def __init__(self):
            super()


    class DatabaseError(Exception):
        def __init__(self):
            super()


    def connect(user, password, host, port, database):
        print("Fallback MariaDB")
        pass


def __read_conf_json(filename: str) -> dict:
    import json
    with open(filename, mode='r') as file:
        return json.load(file)


def connect_to_mariadb_db(conf_filename: str) -> _mariadb.connection:
    if os.path.exists(conf_filename):
        try:
            conn = connect(**__read_conf_json(conf_filename))  # type: _mariadb.connection
            conn.auto_reconnect = True
            return conn
        except Error as e:
            print(f"Error connecting to MariaDB Platform: {e}")
            raise
    else:
        raise FileNotFoundError


def connect_to_sqlite_db(db_filename: str, initiator: Callable = None) -> sqlite3.Connection:
    if os.path.exists(db_filename):
        return sqlite3.connect(db_filename)
    elif initiator is not None:
        connection = sqlite3.connect(db_filename)
        initiator(connection)
        return connection
    else:
        raise FileNotFoundError


def check_connection(connection) -> None:
    try:
        connection.ping()
    except DatabaseError:
        connection.reconnect()
