"""
Copyright or © or Copr. University of Nantes / LS2N (2021)

Contributor:
* Alexis Bitaillou

This software is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use,
modify and/ or redistribute the software under the terms of the CeCILL-C
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info".

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability.

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or
data to be ensured and,  more generally, to use and operate it in the
same conditions as regards security.

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.
"""
import re

import abstract_extractor


class AppCBRStat(abstract_extractor.Extractor):
    @staticmethod
    def extract_from_stat(stat_data):
        pattern_receiver = re.compile(r"(\d+),\s+,\[(\d+)\],\s+Application,\s+CBR \w+,Receiver = (\d+)")
        # pattern_sender = re.compile(r"(\d+),\s+,(\[\d+\]),\s+Application,\s+CBR \w+,Sender = (\d+)")
        # senders = re.findall(pattern_sender, stats_data)
        receivers = re.findall(pattern_receiver, stat_data)
        extracted_cbrs = set()
        for snd in receivers:
            if int(snd[0]) != int(snd[2]):
                extracted_cbrs.add((int(snd[0]), int(snd[1]), int(snd[2])))
        my_cbr_stats = list()
        for ec in extracted_cbrs:
            tmp = re.findall("%s,\s+,\[%s\],\s+Application,\s+" % (ec[2], ec[1]) + re.escape(
                "CBR Server,Total Unicast Messages Received (messages)") + "\s=\s(\d+\.?\d*)", stat_data)
            m_rcv = int(tmp[0]) if len(tmp) != 0 else 0
            tmp = re.findall("%s,\s+,\[%s\],\s+Application,\s+" % (ec[0], ec[1]) + re.escape(
                "CBR Client,Total Unicast Messages Sent (messages)") + "\s=\s(\d+\.?\d*)", stat_data)
            m_snd = int(tmp[0]) if len(tmp) != 0 else 0
            pdr = float(m_rcv) / float(m_snd) if m_snd != 0 else 0
            tmp = re.findall("%s,\s+,\[%s\],\s+Application,\s+" % (ec[2], ec[1]) + re.escape(
                "CBR Server,Average Unicast End-to-End Delay (seconds)") + "\s=\s(\d+\.?\d*)", stat_data)
            delay = float(tmp[0]) if len(tmp) != 0 else 0
            tmp = re.findall("%s,\s+,\[%s\],\s+Application,\s+" % (ec[2], ec[1]) + re.escape(
                "CBR Server,Average Unicast Jitter (seconds)") + "\s=\s(\d+\.?\d*)", stat_data)
            jitter = float(tmp[0]) if len(tmp) != 0 else 0
            tmp = re.findall("%s,\s+,\[%s\],\s+Application,\s+" % (ec[2], ec[1]) + re.escape(
                "CBR Server,Out-of-order drop messages") + "\s=\s(\d+\.?\d*)", stat_data)
            ooo_message = int(tmp[0]) if len(tmp) != 0 else 0
            my_cbr_stats.append(AppCBRStat(ec[0], ec[2], delay, pdr, jitter, ec[1], ooo_message))
        return my_cbr_stats

    def export_csv(self) -> list:
        raise NotImplementedError

    def __init__(self, sender: int, receiver: int, delay: float, pdr: float, jitter: float, source_port: int,
                 ooo_messages: int = 0):
        self.sender = sender
        self.receiver = receiver
        self.delay = delay
        self.pdr = pdr
        self.jitter = jitter
        self.source_port = source_port
        self.ooo_messages = ooo_messages

    def __eq__(self, other):
        if isinstance(other, AppCBRStat):
            return self.sender == other.sender and self.receiver == other.receiver and self.source_port == other.source_port
        return False

    def __hash__(self):
        return (hash(self.sender) ^ (hash(self.receiver) << 1) << 1) ^ hash(self.source_port)

    def export_sql(self, cur, simulation_id: int, autocommit: bool = False):
        cur.execute("INSERT INTO cbr_stats VALUES (?, ?, ?, ?, ?, ?, ?, ?)", (
            simulation_id, self.sender, self.receiver, self.source_port, self.delay, self.pdr, self.jitter,
            self.ooo_messages))
        if autocommit:
            cur.commit()

    @staticmethod
    def setup_sql_db(cur):
        cur.execute(
            "CREATE TABLE cbr_stats (id INTEGER, sender INTEGER, receiver INTEGER, source_port INTEGER, delay REAL, pdr REAL, jitter REAL, ooo_messages INTEGER, PRIMARY KEY(id, sender, receiver, source_port));")
