"""
Copyright or © or Copr. University of Nantes / LS2N (2021)

Contributor:
* Alexis Bitaillou

This software is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use,
modify and/ or redistribute the software under the terms of the CeCILL-C
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info".

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability.

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or
data to be ensured and,  more generally, to use and operate it in the
same conditions as regards security.

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.
"""
import re
import typing

import abstract_extractor


class AppStats(abstract_extractor.Extractor):
    @staticmethod
    def extract_from_stat(stat_data: str):
        apps = AppStats()
        for key in apps.cbr.keys():
            pattern = re.compile(re.escape(key) + r"\s=\s\d+\.?\d*")
            results = re.findall(pattern, stat_data)
            for result in results:
                try:
                    apps.cbr[key] += int(result.split("=")[1])
                except ValueError:
                    apps.cbr[key] += float(result.split("=")[1])
            if key == "CBR Server,Average Unicast End-to-End Delay (seconds)" and len(results) > 0:
                apps.cbr["CBR Server,Average Unicast End-to-End Delay (seconds)"] /= len(results)
            elif key == "CBR Server,Average Unicast Jitter (seconds)" and len(results) > 0:
                apps.cbr["CBR Server,Average Unicast Jitter (seconds)"] /= len(results)
        if apps.cbr["CBR Client,Total Unicast Messages Sent (messages)"] > 0:
            apps.pdr = apps.cbr["CBR Server,Total Unicast Messages Received (messages)"] / apps.cbr[
                "CBR Client,Total Unicast Messages Sent (messages)"]
        return apps

    def __init__(self):
        self.cbr = {
            "CBR Server,Total Unicast Messages Received (messages)": 0,  # type: int
            "CBR Client,Total Unicast Messages Sent (messages)": 0,  # type: int
            "CBR Server,Average Unicast End-to-End Delay (seconds)": 0,  # type: float
            "CBR Server,Out-of-order drop messages": 0,  # type: int
            "CBR Server,Average Unicast Jitter (seconds)": 0  # type: float
        }
        self.pdr = 0.0  # type: float

    def export_sql(self, cur, simulation_id: int, autocommit: bool = False):
        cur.execute("INSERT INTO stats_app VALUES (?, ?, ?, ?, ?, ?, ?);", (
            simulation_id, self.cbr["CBR Server,Average Unicast End-to-End Delay (seconds)"], self.pdr,
            self.cbr["CBR Server,Average Unicast Jitter (seconds)"],
            self.cbr["CBR Client,Total Unicast Messages Sent (messages)"],
            self.cbr["CBR Server,Total Unicast Messages Received (messages)"],
            self.cbr["CBR Server,Out-of-order drop messages"]))
        if autocommit:
            cur.commit()

    def export_csv(self) -> typing.List[int]:
        return [self.cbr["CBR Client,Total Unicast Messages Sent (messages)"],
                self.cbr["CBR Server,Total Unicast Messages Received (messages)"],
                self.cbr["CBR Server,Out-of-order drop messages"],
                self.cbr["CBR Server,Average Unicast End-to-End Delay (seconds)"],
                self.cbr["CBR Server,Average Unicast Jitter (seconds)"],
                self.pdr]

    @staticmethod
    def setup_sql_db(cur):
        cur.execute(
            "CREATE TABLE stats_app (id INTEGER, delay NUMERIC, pdr NUMERIC, jitter NUMERIC, app_send INTEGER, app_recieve INTEGER, app_recieve_ooo INTEGER, PRIMARY KEY (id), FOREIGN KEY (id) REFERENCES setup);")
