"""
Copyright or © or Copr. University of Nantes / LS2N (2021)

Contributor:
* Alexis Bitaillou

This software is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use,
modify and/ or redistribute the software under the terms of the CeCILL-C
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info".

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability.

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or
data to be ensured and,  more generally, to use and operate it in the
same conditions as regards security.

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.
"""
import re
import typing

import abstract_extractor


class NetStats(abstract_extractor.Extractor):
    @staticmethod
    def extract_from_stat(stat_data):
        ns = NetStats()
        for key in ns.network.keys():
            pattern = re.compile(re.escape(key) + r"\s=\s\d+\.?\d*")
            results = re.findall(pattern, stat_data)
            for result in results:
                try:
                    ns.network[key] += int(result.split("=")[1])
                except ValueError:
                    ns.network[key] += float(result.split("=")[1])
        return ns

    def __init__(self):
        self.network = {
            "Data packets sent unicast (packets)": 0,  # type: int
            "Data packets received unicast (packets)": 0,  # type: int
            "Data packets forwarded unicast (packets)": 0,  # type: int
            "Packets dropped due to no route (packets)": 0,  # type: int
            "Packets dropped due to expired TTL (packets)": 0,  # type: int
            "Packets dropped due to queue overflow (packets)": 0  # type: int
        }

    def export_sql(self, cur, simulation_id: int, autocommit: bool = False):
        cur.execute("INSERT INTO stats_net VALUES (?, ?, ?, ?, ?, ?, ?);", (
            simulation_id, self.network["Data packets sent unicast (packets)"],
            self.network["Data packets received unicast (packets)"],
            self.network["Data packets forwarded unicast (packets)"],
            self.network["Packets dropped due to no route (packets)"],
            self.network["Packets dropped due to expired TTL (packets)"],
            self.network["Packets dropped due to queue overflow (packets)"],))
        if autocommit:
            cur.commit()

    def export_csv(self) -> typing.List[int]:
        return [self.network["Data packets sent unicast (packets)"],
                self.network["Data packets received unicast (packets)"],
                self.network["Data packets forwarded unicast (packets)"],
                self.network["Packets dropped due to no route (packets)"],
                self.network["Packets dropped due to expired TTL (packets)"],
                self.network["Packets dropped due to queue overflow (packets)"]]

    @staticmethod
    def setup_sql_db(cur):
        cur.execute(
            "CREATE TABLE stats_net (id INTEGER, net_send INTEGER, net_recieve INTEGER, net_forward INTEGER, drop_no_route INTEGER, drop_expired_ttl INTEGER, drop_overflow INTEGER, PRIMARY KEY (id));")
