"""
Copyright or © or Copr. University of Nantes / LS2N (2021)

Contributor:
* Alexis Bitaillou

This software is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use,
modify and/ or redistribute the software under the terms of the CeCILL-C
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info".

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability.

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or
data to be ensured and,  more generally, to use and operate it in the
same conditions as regards security.

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.
"""
import json
import typing

import abstract_extractor


class QroutingSetting(abstract_extractor.Extractor):
    @staticmethod
    def extract_from_stat(stat_data):
        pass

    def __init__(self, qrouting_setting=None, commit=""):
        self.qrouting_setting = qrouting_setting.copy() if qrouting_setting is not None else dict()
        self.commit = commit

    def export_sql(self, cur, simulation_id: int, autocommit: bool = False):
        extra_param = self.qrouting_setting.copy()
        for key in ["alpha", "explorationPhase", "maxRouteLength", "initReward", "timeout", "periodicUpdate",
                    "timeoutCheckRate", "nbSampleForLatency", "nbSampleEndTrial", "epsilon", "epoch", "predictedValue"]:
            extra_param.pop(key)

        cur.execute("INSERT INTO qrouting VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?);", (
            simulation_id, self.commit, self.qrouting_setting["alpha"],
            self.qrouting_setting["explorationPhase"],
            self.qrouting_setting["maxRouteLength"], self.qrouting_setting["initReward"],
            self.qrouting_setting["timeout"],
            self.qrouting_setting["periodicUpdate"], self.qrouting_setting["timeoutCheckRate"],
            self.qrouting_setting["nbSampleForLatency"], self.qrouting_setting["nbSampleEndTrial"],
            self.qrouting_setting["epoch"], self.qrouting_setting["epsilon"],
            self.qrouting_setting["predictedValue"],
            json.dumps(extra_param, sort_keys=True)))
        if autocommit:
            cur.commit()

    def get_exist_id(self, cur) -> int:
        extra_param = self.qrouting_setting.copy()
        for key in ["alpha", "explorationPhase", "maxRouteLength", "initReward", "timeout", "periodicUpdate",
                    "timeoutCheckRate", "nbSampleForLatency", "nbSampleEndTrial", "epsilon", "epoch", "predictedValue"]:
            extra_param.pop(key)
        cur.execute(
            "SELECT id from qrouting where alpha=? and explorationPhase=? and maxRouteLength=? and initReward=? and timeout=? and periodicUpdate=? and timeoutCheckRate=? and nbSampleForLatency=? and nbSampleEndTrial=? and epoch=? and epsilon=? and predictedValue=? and extra=?;",
            (
                self.qrouting_setting["alpha"],
                self.qrouting_setting["explorationPhase"],
                self.qrouting_setting["maxRouteLength"], self.qrouting_setting["initReward"],
                self.qrouting_setting["timeout"],
                self.qrouting_setting["periodicUpdate"], self.qrouting_setting["timeoutCheckRate"],
                self.qrouting_setting["nbSampleForLatency"], self.qrouting_setting["nbSampleEndTrial"],
                self.qrouting_setting["epoch"], self.qrouting_setting["epsilon"],
                self.qrouting_setting["predictedValue"], json.dumps(extra_param, sort_keys=True)))
        tmp = cur.fetchone()
        return tmp if tmp is not None else -1

    def __getitem__(self, item):
        return self.qrouting_setting[item]

    def export_csv(self) -> typing.List[int]:
        raise NotImplementedError

    @staticmethod
    def setup_sql_db(cur):
        cur.execute(
            "CREATE TABLE qrouting (idQrouting INTEGER auto_increment, alpha REAL, explorationPhase INTEGER, maxRouteLength INTEGER, initReward INTEGER, timeout INTEGER, periodicUpdate INTEGER, timeoutCheckRate INTEGER, nbSampleForLatency INTEGER, nbSampleEndTrial INTEGER, epoch INTEGER, epsilon INTEGER, predictedValue INTEGER, extra TEXT, PRIMARY KEY (idQrouting));")
