#!/usr/bin/env python3
"""
Copyright or © or Copr. University of Nantes / LS2N (2021)

Contributor:
* Alexis Bitaillou

This software is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use,
modify and/ or redistribute the software under the terms of the CeCILL-C
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info".

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability.

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or
data to be ensured and,  more generally, to use and operate it in the
same conditions as regards security.

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.
"""
import itertools
import json
import sqlite3
from typing import Union

from db_connector import _mariadb, connect_to_mariadb_db


def __setup_sql_db(connection: Union[sqlite3.Connection, _mariadb.connection]):
    cur = connection.cursor()
    cur.execute(
        "CREATE TABLE test (id INTEGER auto_increment, scenario TEXT, protocol TEXT, config TEXT, app TEXT, conf TEXT, PRIMARY KEY(id));")
    cur.execute("CREATE TABLE test_status (id INTEGER, status INTEGER, worker_id TEXT, PRIMARY KEY(id));")
    connection.commit()


def generate_all_tests_from_json(filename: str):
    def read_json(_filename) -> dict:
        with open(_filename, 'r', encoding="utf-8") as file:
            text = file.read()
            return json.loads(text)

    data = read_json(filename)
    current_setting = dict()
    config = dict()
    conf = dict()
    app = dict()
    app["CBR"] = dict()
    app["TRAFFIC-GEN"] = dict()
    current_setting["SCENARIO"] = data["SCENARIO"]
    for config_values in itertools.product(*data["CONFIG"].values()):
        for cbr_info in itertools.product(list(data["APP"]["CBR"]["INTERVAL"]), list(data["APP"]["CBR"]["DURATION"]),
                                          data["APP"]["TRAFFIC-GEN"]["INTERVAL"],
                                          data["APP"]["TRAFFIC-GEN"]["DURATION"]):
            app["CBR"]["INTERVAL"] = cbr_info[0]
            app["CBR"]["DURATION"] = cbr_info[1]
            app["TRAFFIC-GEN"]["INTERVAL"] = cbr_info[2]
            app["TRAFFIC-GEN"]["DURATION"] = cbr_info[3]
            for key, value in zip(data["CONFIG"], config_values):
                config[key] = value
            current_setting["APP"] = app
            current_setting["CONFIG"] = config
            for protocole in data["PROTOCOL"]:
                if len(protocole) == 3 and protocole[0] == "QROUTING":
                    current_setting["PROTOCOL"] = protocole
                    all_conf = set()
                    for conf_values in itertools.product(*data["CONF"].values()):
                        for key, value in zip(data["CONF"], conf_values):
                            conf[key] = value
                        if conf["epsilon"] != 0:
                            conf["enableEpoch"] = 0
                        if conf["enableEpoch"] == 0:
                            conf["epoch"] = 0
                        if conf["enableLinearRegression"] == 0:
                            conf["predictedValue"] = 0
                        all_conf.add(json.dumps(conf, sort_keys=True))
                    for my_conf in all_conf:
                        current_setting["CONF"] = json.loads(my_conf)
                        yield current_setting
                else:
                    current_setting["PROTOCOL"] = (protocole[0], "", "")
                    for key in conf.keys():
                        conf[key] = 0
                    current_setting["CONF"] = conf
                    yield current_setting


def exists_row_in_db(connection, values: tuple) -> bool:
    cur = connection.cursor()
    cur.execute('Select id from test where scenario=? and protocol=? and config=? and app=? and conf=?;', values)
    res = cur.fetchone()
    cur.close()
    return res is not None


def fill_db(filename: str, connection: Union[sqlite3.Connection, _mariadb.connection]):
    cur = connection.cursor()
    for generated_test in generate_all_tests_from_json(filename):
        if not exists_row_in_db(conn, (
                generated_test["SCENARIO"], json.dumps(generated_test["PROTOCOL"]),
                json.dumps(generated_test["CONFIG"]),
                json.dumps(generated_test["APP"]), json.dumps(generated_test["CONF"]))):
            cur.execute("INSERT INTO test (scenario, protocol, config, app, conf) VALUES (?, ?, ?, ?, ?);", (
                generated_test["SCENARIO"], json.dumps(generated_test["PROTOCOL"]),
                json.dumps(generated_test["CONFIG"]),
                json.dumps(generated_test["APP"]),
                json.dumps(generated_test["CONF"])))
    cur.close()
    connection.commit()


if __name__ == '__main__':
    # conn = connect_to_sqlite_db("test.db", __setup_sql_db)
    conn = connect_to_mariadb_db("db.json")
    # __setup_sql_db(conn)
    fill_db("test.json", conn)
    conn.close()
