"""
Copyright or © or Copr. University of Nantes / LS2N (2021)

Contributor:
* Alexis Bitaillou

This software is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use,
modify and/ or redistribute the software under the terms of the CeCILL-C
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info".

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability.

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or
data to be ensured and,  more generally, to use and operate it in the
same conditions as regards security.

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.
"""
import sqlite3
from typing import Union

import extractor_app_cbr
import extractor_app_stats
import extractor_net_stats
import extractor_qrouting_setting
from db_connector import _mariadb


class Stats:
    def __init__(self, stat_file: str, current_setting: dict, process_time: float = 0, hash_conf: str = "",
                 qrouting_setting=None, app_content: str = ""):
        with open(stat_file, 'r', encoding='utf8') as file:
            data = file.read()
        self.file = stat_file  # type: str
        self.process_time = process_time
        self.param = current_setting.copy()
        self.qrouting_setting = extractor_qrouting_setting.QroutingSetting(qrouting_setting, self.param["commit"])
        self.app_content = app_content
        self.apps = extractor_app_stats.AppStats.extract_from_stat(data)
        self.ns = extractor_net_stats.NetStats.extract_from_stat(data)
        self.cbrs = extractor_app_cbr.AppCBRStat.extract_from_stat(data)

        self.param["Protocol"] = current_setting["ROUTING-PROTOCOL-IPv4"]  # type: str
        self.param["hash"] = hash_conf  # type: str

    def export_csv(self) -> list:
        return [self.param["Protocol"],
                self.param["SIMULATION-TIME"],
                self.param["SEED"],
                self.param["CBR interval"],
                self.param["CBR duration"],
                self.param["Noise interval"],
                self.param["Noise duration"],
                self.qrouting_setting["alpha"],
                *self.apps.export_csv(),
                *self.ns.export_csv(),
                int(self.process_time),
                self.qrouting_setting["nbSampleForLatency"],
                self.param["hash"]
                ]

    def export_sql(self, conn: Union[sqlite3.Connection, _mariadb.connection]):
        if isinstance(conn, _mariadb.connection):
            cur = conn.cursor()
            cur.execute("START TRANSACTION READ WRITE;")
            cur.execute("LOCK TABLES setup WRITE;")
            cur.execute("SELECT MAX(id) from setup;")
            current_id = cur.fetchone()
            if current_id is None:
                current_id = [0]
        else:
            cur = conn.cursor()
            current_id = cur.execute("SELECT MAX(id) from setup;").fetchone()
            if current_id is None:
                current_id = [0]
        current_id = current_id[0] + 1
        cur.execute("INSERT INTO setup VALUES (?, ?, ?, ?, ?);", (
            current_id, self.param["Protocol"], self.param["SEED"], self.param["SIMULATION-TIME"],
            self.param["scenario"]))
        cur.execute("UNLOCK TABLES;")
        conn.commit()
        if self.param["Protocol"] == "QROUTING":
            self.qrouting_setting.export_sql(cur, current_id)
        cur.execute('INSERT INTO app VALUES (%s, "%s");' % (current_id, self.app_content))
        self.apps.export_sql(cur, current_id)
        self.ns.export_sql(cur, current_id)
        for cbr in self.cbrs:
            cbr.export_sql(cur, current_id)
        cur.close()
        conn.commit()
        conn.close()
