"""
Copyright or © or Copr. University of Nantes / LS2N (2021)

Contributor:
* Alexis Bitaillou

This software is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use,
modify and/ or redistribute the software under the terms of the CeCILL-C
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info".

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability.

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or
data to be ensured and,  more generally, to use and operate it in the
same conditions as regards security.

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.
"""
import os
import random
import sqlite3
import time
from typing import Union

import extractor_app_cbr
from db_connector import _mariadb, check_connection


def __setup_sql_db(connection: Union[sqlite3.Connection, _mariadb.connection]) -> None:
    cur = connection.cursor()
    cur.execute(
        "CREATE TABLE setup (id INTEGER, protocol TEXT, seed INTEGER, duration TEXT, scenario TEXT,  PRIMARY KEY(id AUTOINCREMENT))")
    cur.execute(
        "CREATE TABLE stats_app (id INTEGER, delay NUMERIC, pdr NUMERIC, jitter NUMERIC, app_send INTEGER, app_recieve INTEGER, app_recieve_ooo INTEGER, PRIMARY KEY (id), FOREIGN KEY (id) REFERENCES setup);")
    cur.execute(
        "CREATE TABLE stats_net (id INTEGER, net_send INTEGER, net_recieve INTEGER, net_forward INTEGER, drop_no_route INTEGER, drop_expired_ttl INTEGER, drop_overflow INTEGER, PRIMARY KEY (id), FOREIGN KEY (id) REFERENCES setup);")
    cur.execute("CREATE TABLE app (id INTEGER, app TEXT, PRIMARY KEY (id), FOREIGN KEY (id) REFERENCES setup);")
    cur.execute(
        "CREATE TABLE qrouting (id INTEGER, sha_commit TEXT, alpha NUMERIC, explorationPhase INTEGER, maxRouteLength INTEGER, initReward INTEGER, timeout INTEGER, periodicUpdate INTEGER, timeoutCheckRate INTEGER, nbSampleForLatency INTEGER, nbSampleEndTrial INTEGER, epoch INTEGER, epsilon INTEGER, predictedValue INTEGER, extra TEXT, PRIMARY KEY (id), FOREIGN KEY (id) REFERENCES setup);")
    cur.execute(
        "CREATE TABLE test (id INTEGER, scenario TEXT, protocol TEXT, config TEXT, app TEXT, conf TEXT, PRIMARY KEY(id AUTOINCREMENT));")
    cur.execute("CREATE TABLE test_status (id INTEGER, status INTEGER, worker_id TEXT, PRIMARY KEY(id));")
    extractor_app_cbr.AppCBRStat.setup_sql_db(cur)
    cur.close()
    connection.commit()


def connect_to_sqlite_db(filename: str) -> sqlite3.Connection:
    if os.path.exists(filename):
        return sqlite3.connect(filename)
    else:
        connection = sqlite3.connect(filename)
        __setup_sql_db(connection)
        return connection


def __get_test(connection: Union[sqlite3.Connection, _mariadb.connection], worker_id: str) -> tuple:
    if isinstance(connection, _mariadb.connection):
        check_connection(connection)
        cur = connection.cursor()
        cur.execute("START TRANSACTION READ WRITE;")
    else:
        cur = connection.cursor()
    cur.execute("SELECT * FROM test LEFT JOIN test_status ON test.id = test_status.id WHERE test_status.id IS NULL;")
    next_test = cur.fetchone()
    if next_test is not None:
        cur.execute("INSERT INTO test_status VALUES (?, 0, ?);", (next_test[0], worker_id))
    cur.close()
    connection.commit()
    connection.close()
    return next_test


def lock_test(connection: Union[sqlite3.Connection, _mariadb.connection], worker_id: str) -> tuple:
    next_test = None
    query_done = False
    while not query_done:
        try:
            next_test = __get_test(connection, worker_id)
            query_done = True
        except _mariadb.IntegrityError:
            time.sleep(random.randint(0, 10))
    return next_test


def unlock_test(connection: Union[sqlite3.Connection, _mariadb.connection], work_id: int) -> None:
    cur = connection.cursor()
    cur.execute("DELETE FROM test_status WHERE id=?;", (work_id,))
    cur.close()
    connection.commit()
    connection.close()


def mark_test_completed(connection: Union[sqlite3.Connection, _mariadb.connection], work_id: int) -> None:
    cur = connection.cursor()
    cur.execute("UPDATE test_status SET status=1 WHERE id=?;", (work_id,))
    cur.close()
    connection.commit()
    connection.close()
