"""
Copyright or © or Copr. University of Nantes / LS2N (2021)

Contributor:
* Alexis Bitaillou

This software is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use,
modify and/ or redistribute the software under the terms of the CeCILL-C
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info".

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability.

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or
data to be ensured and,  more generally, to use and operate it in the
same conditions as regards security.

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.
"""
import os.path
import re
import subprocess
import time


def get_branch_name() -> str:
    process = subprocess.run("git status", stdout=subprocess.PIPE, shell=True)
    result = re.findall(r"On\sbranch\s(.+)\n", process.stdout.decode("utf-8"))
    return result[0]


def get_commit_sha() -> str:
    process = subprocess.run("git log -n 1 --abbrev-commit --format=oneline", stdout=subprocess.PIPE, shell=True)
    return process.stdout.decode("utf-8").split(" ")[0]


def check_target_commit(branch: str, commit: str):
    return get_branch_name() == branch and get_commit_sha() == commit


def change_branch(branch: str, source_dir: str) -> bool:
    if get_branch_name() != branch:
        process = subprocess.run("git fetch; git checkout " + branch, cwd=source_dir, stderr=subprocess.STDOUT,
                                 shell=True)
        return process.returncode == 0
    return True


def change_commit(commit: str, source_dir: str) -> bool:
    subprocess.run("git pull", stdout=subprocess.PIPE, shell=True)
    process = subprocess.run("git reset --hard " + commit, cwd=source_dir, stderr=subprocess.STDOUT, shell=True)
    return process.returncode == 0


def compile_qualnet(qualnet_dir: str) -> bool:
    if os.path.exists(qualnet_dir + "/Makefile"):
        process = subprocess.run("make", cwd=qualnet_dir, stderr=subprocess.STDOUT, shell=True)
        return process.returncode == 0
    return False


def make_qualnet_test(config_dir: str, config_file: str, output: str, bin_dir: str, nb_threads: int = 1) -> float:
    cmd = "%s/qualnet -np %s -friendly %s %s" % (bin_dir, nb_threads, config_file, output)
    start_time = time.time()
    result = subprocess.run(cmd, cwd=config_dir, shell=True)
    while result.returncode in [4294967200, 10, 4294967281, -15, -96]:
        result = subprocess.run(cmd, cwd=config_dir)
    return time.time() - start_time
